@@ -112,6 +112,27 @@ struct _xmlSchemaAnnot {
  */
 #define XML_SCHEMAS_ANYATTR_STRICT	3
 
+/**
+ * XML_SCHEMAS_ATTR_USE_PROHIBITED:
+ *
+ * The attribute is prohibited.
+ */
+#define XML_SCHEMAS_ATTR_USE_PROHIBITED 0
+
+/**
+ * XML_SCHEMAS_ATTR_USE_REQUIRED:
+ *
+ * The attribute is required.
+ */
+#define XML_SCHEMAS_ATTR_USE_REQUIRED 1
+
+/**
+ * XML_SCHEMAS_ATTR_USE_OPTIONAL:
+ *
+ * The attribute is optional.
+ */
+#define XML_SCHEMAS_ATTR_USE_OPTIONAL 2
+
 /**
  * XML_SCHEMAS_ATTR_NSDEFAULT:
  *
@@ -200,6 +221,8 @@ struct _xmlSchemaType {
     const xmlChar *baseNs;
     xmlSchemaTypePtr baseType;
     xmlSchemaFacetPtr facets;
+    struct _xmlSchemaType *redef;/* possible redefinitions for the type */
+    int recurse;
 };
 
 /*
@@ -378,6 +401,7 @@ struct _xmlSchema {
     xmlHashTablePtr groupDecl;
     xmlDictPtr      dict;
     void *includes;     /* the includes, this is opaque for now */
+    int preserve;	/* whether to free the document */
 };
 
 XMLPUBFUN void XMLCALL 	xmlSchemaFreeType	(xmlSchemaTypePtr type);
