@@ -1,10 +1,10 @@
 /*
- * xmlIO.h : interface for the I/O interfaces used by the parser
+ * Summary: interface for the I/O interfaces used by the parser
+ * Description: interface for the I/O interfaces used by the parser
  *
- * See Copyright for the status of this software.
- *
- * daniel@veillard.com
+ * Copy: See Copyright for the status of this software.
  *
+ * Author: Daniel Veillard
  */
 
 #ifndef __XML_IO_H__
@@ -133,6 +133,7 @@ struct _xmlParserInputBuffer {
     xmlBufferPtr raw;       /* if encoder != NULL buffer for raw input */
     int	compressed;	    /* -1=unknown, 0=not compressed, 1=compressed */
     int error;
+    unsigned long rawconsumed;/* amount consumed from raw */
 };
 
 
