@@ -1,3 +1,12 @@
+/*
+ * Summary: error handling
+ * Description: the API used to report errors
+ *
+ * Copy: See Copyright for the status of this software.
+ *
+ * Author: Daniel Veillard
+ */
+
 #include <libxml/parser.h>
 
 #ifndef __XML_ERROR_H__
@@ -225,6 +234,9 @@ typedef enum {
     XML_DTD_UNKNOWN_ENTITY, /* 535 */
     XML_DTD_UNKNOWN_ID, /* 536 */
     XML_DTD_UNKNOWN_NOTATION, /* 537 */
+    XML_DTD_STANDALONE_DEFAULTED, /* 538 */
+    XML_DTD_XMLID_VALUE, /* 539 */
+    XML_DTD_XMLID_TYPE, /* 540 */
     XML_HTML_STRUCURE_ERROR = 800,
     XML_HTML_UNKNOWN_TAG, /* 801 */
     XML_RNGP_ANYNAME_ATTR_ANCESTOR = 1000,
@@ -454,6 +466,8 @@ typedef enum {
     XML_XINCLUDE_INCLUDE_IN_INCLUDE, /* 1614 */
     XML_XINCLUDE_FALLBACKS_IN_INCLUDE, /* 1615 */
     XML_XINCLUDE_FALLBACK_NOT_IN_INCLUDE, /* 1616 */
+    XML_XINCLUDE_DEPRECATED_NS, /* 1617 */
+    XML_XINCLUDE_FRAGMENT_ID, /* 1618 */
     XML_CATALOG_MISSING_ATTR = 1650,
     XML_CATALOG_ENTRY_BROKEN, /* 1651 */
     XML_CATALOG_PREFER_VALUE, /* 1652 */
@@ -526,6 +540,15 @@ typedef enum {
     XML_SCHEMAP_REDEFINED_ATTR, /* 1763 */
     XML_SCHEMAP_REDEFINED_NOTATION, /* 1764 */
     XML_SCHEMAP_FAILED_PARSE, /* 1765 */
+    XML_SCHEMAP_UNKNOWN_PREFIX, /* 1766 */
+    XML_SCHEMAP_DEF_AND_PREFIX, /* 1767 */
+    XML_SCHEMAP_UNKNOWN_INCLUDE_CHILD, /* 1768 */
+    XML_SCHEMAP_INCLUDE_SCHEMA_NOT_URI, /* 1769 */
+    XML_SCHEMAP_INCLUDE_SCHEMA_NO_URI, /* 1770 */
+    XML_SCHEMAP_NOT_SCHEMA, /* 1771 */
+    XML_SCHEMAP_UNKNOWN_MEMBER_TYPE, /* 1772 */
+    XML_SCHEMAP_INVALID_ATTR_USE, /* 1723 */
+    XML_SCHEMAP_RECURSIVE, /* 1724 */
     XML_SCHEMAV_NOROOT = 1800,
     XML_SCHEMAV_UNDECLAREDELEM, /* 1801 */
     XML_SCHEMAV_NOTTOPLEVEL, /* 1802 */
